// Demo 7

// Java Arithmetic Operators Application Demo
//
//   by Charles W. Neville, 1998
//   Copyright Charles W. Neville, 1998
//
// Based in part on Deitel and Deitel's Comparison class.

// Illustrates arithmetic operators
//   + (plus), - (minus),
//   * (times), / (divide), % (remainder or modulus)
//
//  User enters two integers, say x and y, in two TextFields, presses
//  Enter key after entering second integer, and application computes and
//  and displays x + y, x - y, x*y, x/y, and x%y on a Panel contained in
//  a Frame.

import java.awt.*;
import java.awt.event.*;  // have to import Java's AWT event classes so
                          // Java will know what an ActionListener and an
                          // ActionEvent are.

import CloseAndExit;  // have to import our CloseAndExit helper class so we
                      // can close our application gracefully when the user
                      // clicks its close box.  class CloseAndExit is contained
                      // in a separate file.

class ArithOpsApplication extends Frame implements ActionListener {

   Label promptA;     // prompt for first TextField
   TextField inputA;  // where user types first number
   Label promptB;     // prompt for second TextField
   TextField inputB;  // where user types second number

   int a, b;  // a and b will hold integer equivalents of numbers
              // the user types in as strings in TextFields inputA
              // and inputB.

   int answer;  // the answer to numeric computations.

   // Constructor for class ArithOpsApplication
   //    Applications always have a constructor method which is called by
   //    by main.  So the constructor is a good place to do initializations.
   public ArithOpsApplication() {

      // An ArithOpsApplication object is a Frame with added structure.
      //   Unfortunately, the default LayoutManager for a Frame is not well
      //   suited a Frame which contains several components and on which we
      //   plan to directly draw.  (The LayoutManager determines how
      //   components are placed on the Frame.)  Thus we have to change the 
      //   LayoutManager to the one used by Applets and Panels.  This is 
      //   arcane, but necessary.

      // Change LayoutManager to one used by Applets and Panels
      setLayout(new FlowLayout() );

      // Create a new instance of the Label class (a new Label object)
      //   which displays the String "Enter an integer", and make
      //   promptA refer to this Label object.
      promptA = new Label("Enter an integer");

      // Create a new instance of the TextField class (a new TextField
      //   object) 9 characters wide, and make inputA refer to this
      //   TextField object.
      inputA = new TextField(9);

      // Do the same for promptB and inputB.
      promptB = new Label("Enter an integer and press Enter");
      inputB = new TextField(9);

      // Add the Labels and TextFields to the application Frame
      add(promptA);
      add(inputA);
      add(promptB);
      add(inputB);

      // Register this object as the ActionListener (ActionEvent processor)
      //   for inputB.  (An action event happens, for example, when the
      //   Enter key is pressed.  Leaving this out causes your application not 
      //   to respond when the user presses the Enter key.
      inputB.addActionListener(this);  // this is a keyword referring to
                                       // the current object, ie. this object. 

      // Register an instance of CloseAndExit as a
      //    window event listener so we can close the Frame and exit
      //    the app gracefully
      addWindowListener( new CloseAndExit() );

      // initialize numeric variables
      //   (we could also have initialized them in the declarations).
      a = 0;  b = 0;  answer = 0;
   }

   // We will actually do the computations in paint just before we
   //   draw the answers on the application Frame
   public void paint( Graphics g ) {

      answer = a + b;  // here + is addition because a and b are ints
      g.drawString(a + " + " + b + " = " + answer, 25, 140);
                                 // here + is String concatenation
                                 // everything is a String (drawString
                                 // sensibly converts ints to Strings).

      answer = a - b;  // note reuse of variable answer.  Efficiency!
      g.drawString(a + " - " + b + " = " + answer, 25, 160);

      answer = a*b;
      g.drawString(a + " * " + b + " = " + answer, 25, 180);

      if (b != 0) {
         answer = a/b;
         g.drawString(a + " / " + b + " = " + answer, 25, 200);
      }
      else {
         answer = 0;
         g.drawString(a + " / " + b + " = ?", 25, 200);
      }
      g.drawString("Note how / gives the integer quotient", 100, 200);

      if (b != 0) {
         answer = a%b;
         g.drawString(a + " % " + b + " = " + answer, 25, 220);
      }
      else {
         answer = 0;
         g.drawString(a + " % " + b + " = ?", 25, 220);
      }
      g.drawString("Note how % gives the integer remainder", 100, 220);
   }

   // This method is what we guarantee to implement when we declare that
   // we implement the ActionListener interface.  When the user presses the
   // Enter key while the cursor is in the inputB TextField, an
   // actionPerformed message will be sent to us.  We will respond by
   // carrying out (running the code in) our actionPerformed method.  We
   // could put any code in here we want, but since we want to convert
   // the user's String input to ints so we can do arithmetic on them,
   // that is what we shall do.  As explained above, we will do the
   // actual calculations in the paint method.
   public void actionPerformed( ActionEvent e ) {

       a = Integer.parseInt(inputA.getText());  // convert String contents
                                          // of inputA TextField to an int.

       b = Integer.parseInt(inputB.getText());

       // force Frame to be repainted so results will be painted.
       repaint();
   }

   public static void main(String[] args) {

     ArithOpsApplication a;

     // Construct an instance of the ArithOpsApplication class
     // (an ArithOpsApplication object).
     a = new ArithOpsApplication();

     // An ArithOpsApplication object is a Frame with additional structure.
     // Set the Frame's size to 350 pixels by 400 pixels,
     //   and make the Frame visible (show the Frame).
     a.setSize(375, 400);
     a.setVisible(true);
   }
}
